package com.polymorphiccoder.backitup;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

/**
 * This is simple java program which backs up the contents of the folder/
 * file in the program argument to a zip file.
 * @author Pradeep
 */
public class DataBacker {
	 
	private static final int DATA_BUFFER_SIZE = 2048;

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// args[0] - File/ folder to be backed up as a zip
		// args[1] - Absolute path of the zip file ending with its name and .zip
		// extension
		
		try {
			FileOutputStream dest = new FileOutputStream(args[1]);
			ZipOutputStream out = new ZipOutputStream(new BufferedOutputStream(
					dest));
			byte data[] = new byte[DATA_BUFFER_SIZE];
			File f = new File(args[0]);
			System.out.println("Started backing up of " + f.getCanonicalPath() + "...");
			addZipFileEntries(out, data, f);
			out.close();
			System.out.println("Done!");
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Method called recursively for adding the contents of each directory to the
	 * the zip file which contains the backed up data.
	 */
	private static void addZipFileEntries(ZipOutputStream out, byte[] data, File f)
			throws FileNotFoundException, IOException {
		if (f.isDirectory()) {
			File files[] = f.listFiles();
			for (int i = 0; i < files.length; i++) {
				if (files[i].isFile()) {
					addFileToZip(out, data, files[i]);
				} else {
					addZipFileEntries(out, data, files[i]);
				}
			}
		} else {
			addFileToZip(out, data, f);
		}
	}
	
	/**
	 * Adds each file to the zip. 
	 */
	private static void addFileToZip(ZipOutputStream out, byte[] data,
			File file) throws FileNotFoundException, IOException {
		BufferedInputStream origin = null;
		int count;
		FileInputStream fi = new FileInputStream(file);
		origin = new BufferedInputStream(fi, DATA_BUFFER_SIZE);
		ZipEntry entry = new ZipEntry(file.getCanonicalPath());
		out.putNextEntry(entry);
		System.out.println("Adding the file " + file.getName() + " to the zip...");
		while ((count = origin.read(data, 0, DATA_BUFFER_SIZE)) != -1) {
			out.write(data, 0, count);
		}
		origin.close();
	}
}
